// =============================================================================
// Copyright (C) MACNICA inc.
// =============================================================================
// FILE NAME	  : fpga_top.v 
// DEPARTMENT	  : MACNICA Strategic Technology Group
// AUTHOR		  : Youhei Nagano
// AUTHOR'S EMAIL : Nagano-y@macnica.co.jp
// =============================================================================
// KEYWORDS :  PRJ-000114
// =============================================================================
// PURPOSE	:  Scan Conversion IP Reference Design
// =============================================================================
// COMMENTS : Registered by Nagano , 100511
// =============================================================================

module i2c_ctl_wrp (
	clk				,
	rst_n			,
	srst			,
	i2cm_scl_i		,
	i2cm_scl_o		,
	i2cm_scl_en_o	,
	i2cm_sda_i		,
	i2cm_sda_o		,
	i2cm_sda_en_o	,
	init_end_o
	);

/****************************************************************/
/*****					parameter definition				*****/
/****************************************************************/

/****************************************************************/
/*****					port declaration					*****/
/****************************************************************/
	input						clk;
	input						rst_n;
	input						srst;

	input						i2cm_scl_i;
	output						i2cm_scl_o;
	output						i2cm_scl_en_o;
	input						i2cm_sda_i;
	output						i2cm_sda_o;
	output						i2cm_sda_en_o;
	output						init_end_o;
/****************************************************************/
/*****					wire signal assignment				*****/
/****************************************************************/
// ==========
// i2c init
// ==========
	wire	[6:0]						i2cm_control_reg;	
	wire	[9:0]						i2cm_dev_sel;		
	wire	[23:0]						i2cm_w_r_adr;		
	wire	[23:0]						i2cm_b_size;		
	wire	[10:0]						i2cm_div;
	wire								i2cm_start; 		
	wire	[7:0]						i2cm_wt_data;		
	
	wire								i2cm_busy;			
	wire								i2cm_error; 		
	wire	[23:0]						i2cm_rem_num;		
	wire	[7:0]						i2cm_rd_data;		
	wire								i2cm_rd_d_valid;	
	wire								i2cm_wt_d_req;		

	wire	[6:0]						i2cm_di_control_reg;  
	wire	[9:0]						i2cm_di_dev_sel;	  
	wire	[23:0]						i2cm_di_w_r_adr;	  
	wire	[23:0]						i2cm_di_b_size; 	  
	wire								i2cm_di_start;		  
	wire	[7:0]						i2cm_di_wt_data;	  
	wire								i2c_di_err;
/****************************************************************/
/*****				lower module declaration				*****/
/****************************************************************/
// ==========
// i2c init
// ==========
	i2c_master_top
		# (
			.DIV_COUNT_BIT_W		( 11				) 
		)
		uI2C_M (
			.clk					( clk				) ,
			.rst_n					( rst_n				) ,
			.srst					( srst				) ,

			.scl_i					( i2cm_scl_i		) ,
			.scl_o					( i2cm_scl_o		) ,
			.scl_en_o				( i2cm_scl_en_o		) ,
			.sda_i					( i2cm_sda_i		) ,
			.sda_o					( i2cm_sda_o		) ,
			.sda_en_o				( i2cm_sda_en_o		) ,

			.control_reg_i			( i2cm_control_reg	) ,
			.dev_sel_i				( i2cm_dev_sel		) ,
			.w_r_adr_i				( i2cm_w_r_adr		) ,
			.b_size_i				( i2cm_b_size		) ,
			.div_i					( i2cm_div			) ,
			.start_i				( i2cm_start		) ,

			.busy_o 				( i2cm_busy			) ,
			.error_o				( i2cm_error		) ,
			.rem_num_o				( i2cm_rem_num		) ,

			.rd_data_o				( i2cm_rd_data		) ,
			.rd_d_valid_o			( i2cm_rd_d_valid	) ,
			.wt_data_i				( i2cm_wt_data		) ,
			.wt_d_req_o 			( i2cm_wt_d_req		) 
		);
	
	assign i2cm_control_reg = i2cm_di_control_reg	;
	assign i2cm_dev_sel 	= i2cm_di_dev_sel		;
	assign i2cm_w_r_adr 	= i2cm_di_w_r_adr		;
	assign i2cm_b_size		= i2cm_di_b_size		;
	assign i2cm_start		= i2cm_di_start 		;
	assign i2cm_wt_data 	= i2cm_di_wt_data		;
	assign i2cm_div 		= 10'b00_0010_0000		;

	i2c_init_ctl
					u_I2C_INIT_CTL (
		.clk					( clk ),
		.rst_n					( rst_n ),
		.srst					( srst ),
		
		.wt_d_req_i 			( i2cm_wt_d_req ),
		.rd_data_i				( i2cm_rd_data ),
		.rd_d_valid_i			( i2cm_rd_d_valid ),
		
		.busy_i 				( i2cm_busy ),
		.error_i				( i2cm_error ),
		
		.control_reg_o			( i2cm_di_control_reg ),
		.dev_sel_o				( i2cm_di_dev_sel ),
		.w_r_adr_o				( i2cm_di_w_r_adr ),
		.b_size_o				( i2cm_di_b_size ),
		.start_o				( i2cm_di_start ),
		.wt_data_o				( i2cm_di_wt_data ),
		
		.init_end_o 			( init_end_o ),
		.err_status_o			( i2c_di_err )
	);

endmodule