// =============================================================================
// Copyright (C) MACNICA inc.
// =============================================================================
// FILE NAME	: avb_frm_wt_fifo_bank.v
// DEPARTMENT	: MACNICA Strategic Technology Group
// AUTHOR		: Ryosuke Yamamoto
// DATE			: 03/10/2014
// =============================================================================
// DESCRIPTION	: 
//
// =============================================================================
// REVISION HISTORY	:
//	v1.0	: Initial version release
//
// =============================================================================
// PARAMETERS	:
//
// =============================================================================
// I/O PORTS	:
//
// =============================================================================
`timescale 1ps/1ps
`default_nettype none

module	avb_frm_wt_fifo_bank (
	clk				,
	rst_n			,
	srst			,

	acc_adr_i		,
	acc_adr_valid_i	,

	fifo_wt_req_i	,
	fifo_wt_end_i	,
	fifo_wt_sel_o	,

	fifo_rd_end_i	,
	acc_adr_o		,
	fifo_rd_sel_o	
) ;
	
// =============================================================================
// PARAMETER DEFINITION
// =============================================================================

	// ---------------------------------------------------------------------
	// Below parameters have to be defined from upper module
	// ---------------------------------------------------------------------

	// ---------------------------------------------------------------------
	// Please do not change the following parameters
	// ---------------------------------------------------------------------
	parameter	AVMM_ADR_BIT	= 27 ;
	localparam	FIFO_BANK_NUM	= 2 ;

// =============================================================================
// PORT DECLARATION
// =============================================================================

	input	wire							clk				;
	input	wire							rst_n			;
	input	wire							srst			;

	input	wire	[  AVMM_ADR_BIT-1 : 0 ]	acc_adr_i		;
	input	wire							acc_adr_valid_i	;

	input	wire							fifo_wt_req_i	;
	input	wire							fifo_wt_end_i	;
	output	wire	[ FIFO_BANK_NUM-1 : 0 ]	fifo_wt_sel_o	;

	input	wire							fifo_rd_end_i	;
	output	wire	[  AVMM_ADR_BIT-1 : 0 ]	acc_adr_o		;
	output	wire	[ FIFO_BANK_NUM-1 : 0 ]	fifo_rd_sel_o	;


// =============================================================================
// REG / WIRE DECLARATION
// =============================================================================

	reg								wt_point_ff		;
	reg								rd_point_ff		;
	reg		[  AVMM_ADR_BIT-1 : 0 ]	adr_0_ff		;
	reg		[  AVMM_ADR_BIT-1 : 0 ]	adr_1_ff		;
	reg		[ FIFO_BANK_NUM-1 : 0 ]	bank_flag_ff	;
	reg								fifo_wt_req_ff	;
	reg								wt_point_lat_ff	;
	reg		[ FIFO_BANK_NUM-1 : 0 ]	fifo_wt_sel_ff	;
	reg								fifo_rd_req_ff	;
	reg		[  AVMM_ADR_BIT-1 : 0 ]	acc_adr_ff		;
	reg		[ FIFO_BANK_NUM-1 : 0 ]	fifo_rd_sel_ff	;


// =============================================================================
// FUNCTION DESCRIPTION
// =============================================================================

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			wt_point_ff	<= 1'b0 ;
		end
		else if ( srst ) begin
			wt_point_ff	<= 1'b0 ;
		end
		else begin
			if ( fifo_wt_end_i ) begin
				wt_point_ff	<= ~wt_point_ff ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			rd_point_ff	<= 1'b0 ;
		end
		else if ( srst ) begin
			rd_point_ff	<= 1'b0 ;
		end
		else begin
			if ( fifo_rd_end_i ) begin
				rd_point_ff	<= ~rd_point_ff ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			adr_0_ff	<= { AVMM_ADR_BIT{1'b0} } ;
			adr_1_ff	<= { AVMM_ADR_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			adr_0_ff	<= { AVMM_ADR_BIT{1'b0} } ;
			adr_1_ff	<= { AVMM_ADR_BIT{1'b0} } ;
		end
		else begin
			if ( acc_adr_valid_i ) begin
				adr_0_ff	<= ( ~wt_point_lat_ff ) ?	acc_adr_i	: adr_0_ff ;
				adr_1_ff	<= (  wt_point_lat_ff ) ?	acc_adr_i	: adr_1_ff ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			bank_flag_ff	<= { FIFO_BANK_NUM{1'b0} } ;
		end
		else if ( srst ) begin
			bank_flag_ff	<= { FIFO_BANK_NUM{1'b0} } ;
		end
		else begin
			case ( bank_flag_ff ) 
				2'b00	: begin
					if ( fifo_wt_end_i ) begin
						bank_flag_ff	<= { wt_point_ff , ~wt_point_ff } ;
					end
				end

				2'b01	: begin
					if ( fifo_wt_end_i && fifo_rd_end_i ) begin
						bank_flag_ff	<= ~bank_flag_ff ;
					end
					else if ( fifo_wt_end_i && ~fifo_rd_end_i ) begin
						bank_flag_ff	<= 2'b11 ;
					end
					else if ( ~fifo_wt_end_i && fifo_rd_end_i ) begin
						bank_flag_ff	<= 2'b00 ;
					end
				end

				2'b10	: begin
					if ( fifo_wt_end_i && fifo_rd_end_i ) begin
						bank_flag_ff	<= ~bank_flag_ff ;
					end
					else if ( fifo_wt_end_i && ~fifo_rd_end_i ) begin
						bank_flag_ff	<= 2'b11 ;
					end
					else if ( ~fifo_wt_end_i && fifo_rd_end_i ) begin
						bank_flag_ff	<= 2'b00 ;
					end
				end

				2'b11	: begin
					if ( fifo_rd_end_i ) begin
						bank_flag_ff	<= { ~rd_point_ff , rd_point_ff } ;
					end
				end
			endcase
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			fifo_wt_req_ff	<= 1'b0 ;
			wt_point_lat_ff	<= 1'b0 ;
		end
		else if ( srst ) begin
			fifo_wt_req_ff	<= 1'b0 ;
			wt_point_lat_ff	<= 1'b0 ;
		end
		else begin
			if ( fifo_wt_req_i ) begin
				fifo_wt_req_ff	<= 1'b1 ;
				wt_point_lat_ff	<= wt_point_ff ;
			end
			else if ( |fifo_wt_sel_ff ) begin
				fifo_wt_req_ff	<= 1'b0 ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			fifo_wt_sel_ff	<= { FIFO_BANK_NUM{1'b0} } ;
		end
		else if ( srst ) begin
			fifo_wt_sel_ff	<= { FIFO_BANK_NUM{1'b0} } ;
		end
		else begin
			if ( fifo_wt_end_i ) begin
				fifo_wt_sel_ff	<= { FIFO_BANK_NUM{1'b0} } ;
			end
			else if ( fifo_wt_req_ff && ( ( ~bank_flag_ff[1] ==  wt_point_ff ) ||
			                              ( ~bank_flag_ff[0] == ~wt_point_ff ) ) ) begin
				fifo_wt_sel_ff	<= { wt_point_ff , ~wt_point_ff }  ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			acc_adr_ff		<= { AVMM_ADR_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			acc_adr_ff		<= { AVMM_ADR_BIT{1'b0} } ;
		end
		else begin
			if ( |bank_flag_ff ) begin
				acc_adr_ff		<= ( rd_point_ff ) ?	adr_1_ff :	adr_0_ff ;
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			fifo_rd_sel_ff	<= { FIFO_BANK_NUM{1'b0} } ;
		end
		else if ( srst ) begin
			fifo_rd_sel_ff	<= { FIFO_BANK_NUM{1'b0} } ;
		end
		else begin
			if ( fifo_rd_end_i ) begin
				fifo_rd_sel_ff	<= { FIFO_BANK_NUM{1'b0} } ;
			end
			else begin
				fifo_rd_sel_ff	<= { ( bank_flag_ff[1] & rd_point_ff ) , ( bank_flag_ff[0] & ~rd_point_ff ) } ;
			end
		end
	end

	//---------------//
	// output signal //
	//---------------//
	assign	fifo_wt_sel_o	= fifo_wt_sel_ff ;
	assign	acc_adr_o		= acc_adr_ff ;
	assign	fifo_rd_sel_o	= fifo_rd_sel_ff ;


endmodule

`default_nettype wire
