// =============================================================================
// Copyright (C) MACNICA inc.
// =============================================================================
// FILE NAME	: avb_frm_wt_st_in.v
// DEPARTMENT	: MACNICA Strategic Technology Group
// AUTHOR		: Ryosuke Yamamoto
// DATE			: 03/10/2014
// =============================================================================
// DESCRIPTION	: 
//
// =============================================================================
// REVISION HISTORY	:
//	v1.0	: Initial version release
//
// =============================================================================
// PARAMETERS	:
//
// =============================================================================
// I/O PORTS	:
//
// =============================================================================
`timescale 1ps/1ps
`default_nettype none

module	avb_frm_wt_st_in (
	clk				,
	rst_n			,
	srst			,

	valid_i			,
	startofpacket_i	,
	endofpacket_i	,
	data_i			,
	ready_o			,

	fifo_full_i		,
	fifo_afull_i	,
	fifo_we_o		,
	fifo_wdata_o	
) ;
	
// =============================================================================
// PARAMETER DEFINITION
// =============================================================================

	// ---------------------------------------------------------------------
	// Below parameters have to be defined from upper module
	// ---------------------------------------------------------------------

	// ---------------------------------------------------------------------
	// Please do not change the following parameters
	// ---------------------------------------------------------------------
	parameter	ST_DATA_BIT		= 32 ;					// Avalon-ST data bit width
	parameter	FIFO_DATA_BIT	= ST_DATA_BIT * 2 + 2 ;	// FIFO data bit width


// =============================================================================
// PORT DECLARATION
// =============================================================================

	input	wire							clk				;
	input	wire							rst_n			;
	input	wire							srst			;

	// TSE MAC format
	input	wire							valid_i			;
	input	wire							startofpacket_i	;
	input	wire							endofpacket_i	;
	input	wire	[   ST_DATA_BIT-1 : 0 ]	data_i			;
	output	wire							ready_o			;

	input	wire							fifo_full_i		;
	input	wire							fifo_afull_i	;
	output	wire							fifo_we_o		;
	output	wire	[ FIFO_DATA_BIT-1 : 0 ]	fifo_wdata_o	;


// =============================================================================
// REG / WIRE DECLARATION
// =============================================================================

	wire							sop				;
	wire							eop				;
	reg								data_valid_ff	;
	reg								we_ff			;
	reg		[ FIFO_DATA_BIT-1 : 0 ]	data_ff			;


// =============================================================================
// FUNCTION DESCRIPTION
// =============================================================================

	//------------------------//
	// Avalon-ST flow control //
	//------------------------//
	assign	ready_o		= ~fifo_full_i ;
	
	assign	sop	= valid_i & startofpacket_i ;
	assign	eop	= valid_i & endofpacket_i ;

	//--------------//
	// FIFO control //
	//--------------//
	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			data_valid_ff	<= 1'b0 ;
		end
		else if ( srst ) begin
			data_valid_ff	<= 1'b0 ;
		end
		else begin
			if ( !fifo_full_i ) begin
				if ( eop ) begin
					data_valid_ff	<= 1'b0 ;
				end
				else if ( valid_i ) begin
					data_valid_ff	<= ~data_valid_ff ;
				end
			end
		end
	end

	always @ ( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			we_ff	<= 1'b0 ;
			data_ff	<= { FIFO_DATA_BIT{1'b0} } ;
		end
		else if ( srst ) begin
			we_ff	<= 1'b0 ;
			data_ff	<= { FIFO_DATA_BIT{1'b0} } ;
		end
		else begin
			we_ff	<= ( ( !fifo_full_i & data_valid_ff & valid_i ) | 
					     ( !fifo_full_i & valid_i & eop ) ) ;
			if ( !fifo_full_i & valid_i ) begin
				data_ff	<= ( !data_valid_ff ) ?	{ sop , eop , data_i , data_ff[ST_DATA_BIT-1:0] } :
												{ data_ff[FIFO_DATA_BIT-1] , eop , data_ff[FIFO_DATA_BIT-3:ST_DATA_BIT] , data_i } ;
			end
		end
	end

	assign	fifo_we_o		= we_ff ;
	assign	fifo_wdata_o	= data_ff ;


endmodule

`default_nettype wire
